unit fmBaseDBgrid;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, fmBaseMDIchild, ExtCtrls, StdCtrls, Buttons,
  FMTBcd, DB, Provider, DBClient, SqlExpr, Grids, DBGrids;

type
  TfrmBaseDBgrid = class(TfrmBaseMDIchild)
    btnNew: TBitBtn;
    btnEdit: TBitBtn;
    btnDelete: TBitBtn;
    pnlLeft: TPanel;
    pnlRight: TPanel;
    splMain: TSplitter;
    sqryData: TSQLQuery;
    cdsData: TClientDataSet;
    dspData: TDataSetProvider;
    dsData: TDataSource;
    pnlRightBottom: TPanel;
    btnPost: TBitBtn;
    btnCancel: TBitBtn;
    btnRefresh: TBitBtn;
    btnOk: TBitBtn;
    btnCancelForm: TBitBtn;
    dgData: TDBGrid;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cdsDataAfterPost(DataSet: TDataSet);
    procedure cdsDataReconcileError(DataSet: TCustomClientDataSet;
      E: EReconcileError; UpdateKind: TUpdateKind;
      var Action: TReconcileAction);
    procedure btnNewClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnEditClick(Sender: TObject);
    procedure cdsDataBeforeClose(DataSet: TDataSet);
    procedure cdsDataAfterEdit(DataSet: TDataSet);
    procedure cdsDataAfterInsert(DataSet: TDataSet);
    procedure cdsDataAfterCancel(DataSet: TDataSet);
    procedure btnPostClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnRefreshClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    parent_id:integer;  //to be set by child after called by parent
  end;

var
  frmBaseDBgrid: TfrmBaseDBgrid;

implementation

uses dmMain, fmBase, strUtils;

{$R *.dfm}

procedure TfrmBaseDBgrid.FormCreate(Sender: TObject);
begin
  inherited;

  {Link back to any parent}
  if (Sender is TForm) and ((Sender as TForm).owner is TfrmBase) then
  begin
    parent_id:=((Sender as TForm).owner as TfrmBase).id;
    sqryData.ParamByName('parent_id').AsInteger:=parent_id;

    pnlTop.caption:=' '+((Sender as TForm).owner as TfrmBase).name+pnlTop.caption;
  end;

  {Open the main dataset}
  cdsData.Open;
end;

procedure TfrmBaseDBgrid.FormDestroy(Sender: TObject);
begin
  {Close the main dataset}
  cdsData.Close;

  inherited;
end;

procedure TfrmBaseDBgrid.cdsDataAfterPost(DataSet: TDataSet);
begin
  inherited;

  {Save the changes after every post}
  btnPost.Enabled:=False;
  btnCancel.Enabled:=False;

  (DataSet as TCustomClientDataSet).ApplyUpdates(0);
  //todo: report differences if returns >0 errors
end;

procedure TfrmBaseDBgrid.cdsDataAfterCancel(DataSet: TDataSet);
begin
  inherited;
  btnPost.Enabled:=False;
  btnCancel.Enabled:=False;
end;

procedure TfrmBaseDBgrid.cdsDataReconcileError(
  DataSet: TCustomClientDataSet; E: EReconcileError;
  UpdateKind: TUpdateKind; var Action: TReconcileAction);
begin
  inherited;
  //todo: resolve differences
  showmessage(E.message);
  Action:=raCancel;
end;

procedure TfrmBaseDBgrid.btnNewClick(Sender: TObject);
begin
  inherited;
  cdsData.Insert;
end;

procedure TfrmBaseDBgrid.btnDeleteClick(Sender: TObject);
begin
  inherited;
  if messageDlg('Confirm deletion?',mtConfirmation,[mbYes,mbNo],0)=mrYes then
    cdsData.Delete;
end;

procedure TfrmBaseDBgrid.btnEditClick(Sender: TObject);
begin
  inherited;
  cdsData.Edit;
end;

procedure TfrmBaseDBgrid.cdsDataBeforeClose(DataSet: TDataSet);
begin
  inherited;
  if cdsData.State in [dsEdit,dsInsert] then
    cdsData.Post;
end;

procedure TfrmBaseDBgrid.cdsDataAfterEdit(DataSet: TDataSet);
begin
  inherited;

  btnPost.Enabled:=True;
  btnCancel.Enabled:=True;
end;

procedure TfrmBaseDBgrid.cdsDataAfterInsert(DataSet: TDataSet);
begin
  inherited;

  btnPost.Enabled:=True;
  btnCancel.Enabled:=True;
end;



procedure TfrmBaseDBgrid.btnPostClick(Sender: TObject);
begin
  inherited;
  cdsData.Post;
  dgData.SetFocus;
end;

procedure TfrmBaseDBgrid.btnCancelClick(Sender: TObject);
begin
  inherited;
  cdsData.Cancel;
  dgData.SetFocus;
end;

procedure TfrmBaseDBgrid.btnRefreshClick(Sender: TObject);
begin
  inherited;
  cdsData.Refresh;
end;

procedure TfrmBaseDBgrid.FormShow(Sender: TObject);
var
  modal:boolean;
begin
  inherited;

  modal:=(formStyle=fsNormal);

  btnCancelForm.Visible:=modal;
  btnOk.Visible:=modal;
end;



end.
